@@ -10,12 +10,15 @@
 use 5.014;
 use utf8;
 package App::SpreadRevolutionaryDate::Config;
-$App::SpreadRevolutionaryDate::Config::VERSION = '0.09';
+$App::SpreadRevolutionaryDate::Config::VERSION = '0.10';
 # ABSTRACT: Companion class of L<App::SpreadRevolutionaryDate>, to handle configuration file and command line arguments, subclass of L<AppConfig>.
 
 use Moose;
 use MooseX::NonMoose;
+
 extends 'AppConfig';
+
+use Getopt::Long;
 use AppConfig qw(:argcount);
 use File::HomeDir;
 use Class::Load ':all';
@@ -25,16 +28,32 @@ use namespace::clean;
 sub new {
   my ($class, $filename) = @_;
 
+  # Backup command line arguments to be consumed twice
+  my @orig_argv = @ARGV;
+
+  # Parse command line only parameters
+  my $config_first = Getopt::Long::Parser->new;
+  $config_first->configure('pass_through');
+
+  if ($filename) {
+    $config_first->getoptions("version|v" => sub { say $App::SpreadRevolutionaryDate::Config::VERSION; exit 0; },
+                              "help|h|?" => \&usage);
+  } else {
+    $config_first->getoptions("version|v" => sub { say $App::SpreadRevolutionaryDate::Config::VERSION; exit 0; },
+                              "help|h|?" => \&usage,
+                              "conf|c=s" => \$filename);
+  }
+
   # If filename is not a file path but a GLOB or an opend filehandle
   # we'll need to rewind it to the beginning before reading it twice
   my $file_start;
   $file_start = tell $filename if $filename && ref($filename);
 
-  # Backup command line arguments to be consumed twice
-  my @orig_argv = @ARGV;
-
   # Find targets
   my $config_targets = AppConfig::new($class, {CREATE => 1, ERROR => sub {}},
+                                      'conf' => {ARGCOUNT => ARGCOUNT_ONE, ALIAS => 'c'},
+                                      'version' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'v'},
+                                      'help' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'h|?'},
                                       'targets' => {ARGCOUNT => ARGCOUNT_LIST, ALIAS => 'tg'},
                                       'msgmaker' => {ARGCOUNT => ARGCOUNT_ONE, ALIAS => 'mm'},
                                       'test' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'no|n'},
@@ -44,6 +63,8 @@ sub new {
                                       'mastodon' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'm'},
                                       'freenode' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'f'});
   $config_targets->parse_file($filename);
+  # Rewind command line arguments and process them
+  @ARGV = @orig_argv;
   $config_targets->parse_command_line;
 
   # Add targets defined with targets option
@@ -54,7 +75,7 @@ sub new {
   foreach my $potential_target (keys %potential_targets) {
     next unless $potential_targets{$potential_target};
     next if $potential_target =~ /_/;
-    if ($potential_target !~ /^(?:acab|test|locale|targets|msgmaker)$/) {
+    if ($potential_target !~ /^(?:acab|test|locale|targets|msgmaker|conf|version|help)$/) {
       push @targets, $potential_target;
     }
   }
@@ -84,6 +105,10 @@ sub new {
       my $target_meta_attribute_argcount = $target_meta_attribute_type =~ /ArrayRef/ ? ARGCOUNT_LIST : $target_meta_attribute_type =~ /HashRef/ ? ARGCOUNT_HASH : $target_meta_attribute_type =~ /Bool/ ? ARGCOUNT_NONE : ARGCOUNT_ONE;
       $target_attributes{lc($target) . '_' . $target_meta_attribute->name} = { ARGCOUNT => $target_meta_attribute_argcount };
       $target_attributes{lc($target)} = { ARGCOUNT => ARGCOUNT_NONE };
+
+      if ($target_meta_attribute->has_default) {
+        $target_attributes{lc($target) . '_' . $target_meta_attribute->name}{DEFAULT} = $target_meta_attribute->default;
+      }
     }
   }
 
@@ -99,16 +124,24 @@ sub new {
   die "Cannot found msgmaker meta class $msgmaker_class for msgmaker $msgmaker: $@\n" if $@;
   foreach my $msgmaker_meta_attribute ($msgmaker_meta->get_all_attributes) {
     next if $msgmaker_meta_attribute->name eq 'locale';
+
     my $msgmaker_meta_attribute_type = $msgmaker_meta_attribute->type_constraint;
     my $msgmaker_meta_attribute_argcount = $msgmaker_meta_attribute_type =~ /ArrayRef/ ? ARGCOUNT_LIST : $msgmaker_meta_attribute_type =~ /HashRef/ ? ARGCOUNT_HASH : $msgmaker_meta_attribute_type =~ /Bool/ ? ARGCOUNT_NONE : ARGCOUNT_ONE;
     $msgmaker_attributes{lc($msgmaker) . '_' . $msgmaker_meta_attribute->name} = { ARGCOUNT => $msgmaker_meta_attribute_argcount };
     $msgmaker_attributes{lc($msgmaker)} = { ARGCOUNT => ARGCOUNT_NONE };
+
+    if ($msgmaker_meta_attribute->has_default) {
+      $msgmaker_attributes{lc($msgmaker) . '_' . $msgmaker_meta_attribute->name}{DEFAULT} = $msgmaker_meta_attribute->default;
+    }
   }
 
   # Build actual instance
   my $self = AppConfig::new($class,
     %target_attributes,
     %msgmaker_attributes,
+    'conf' => {ARGCOUNT => ARGCOUNT_ONE, ALIAS => 'c'},
+    'version' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'v'},
+    'help' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'h|?'},
     'targets' => {ARGCOUNT => ARGCOUNT_LIST, ALIAS => 'tg'},
     'msgmaker' => {ARGCOUNT => ARGCOUNT_ONE, ALIAS => 'mm', default => 'RevolutionaryDate'},
     'test' => {ARGCOUNT => ARGCOUNT_NONE, ALIAS => 'no|n'},
@@ -149,7 +182,7 @@ sub new {
   foreach my $potential_target (keys %confvars) {
     next unless $confvars{$potential_target};
     next if $potential_target =~ /_/;
-    if ($potential_target !~ /^(?:acab|test|locale|targets|msgmaker)$/) {
+    if ($potential_target !~ /^(?:acab|test|locale|targets|msgmaker|conf|version|help)$/) {
       push @targets, $potential_target;
       $self->targets($potential_target);
     }
@@ -252,6 +285,40 @@ sub get_msgmaker_arguments {
 }
 
 
+sub usage {
+ print << "USAGE";
+Usage: $0 <OPTIONS>
+  with <OPTIONS>:
+    --conf|-c <file>: path to configuration file (default: ~/.config/spread-revolutionary-date/spread-revolutionary-date.conf or ~/.spread-revolutionary-date.conf)'
+    --version|-v': print out version
+    --help|-h|-?': print out this help
+    --targets|-tg <target_1> [--targets|-tg <target_2> […--targets|-tg <target_n>]]': define targets (default: twitter, mastodon, freenode)
+    --msgmaker|-mm <MsgMakerClass>: define message maker (default: RevolutionaryDate)
+    --locale|-l <en|fr>: define locale (default: fr)
+    --test|--no|-n: do not spread, just print out message or spread to test channels for Freenode
+    --acab|-a: DEPRECATED, use --revolutionarydate-acab
+    --twitter|-t: DEPRECATED, use --targets=twitter
+    --mastodon|-m: DEPRECATED, use --targets=mastodon
+    --freenode|-f: DEPRECATED, use --targets=freenode
+    --twitter_consumer_key|-tck <key>: define Twitter consumer key
+    --twitter_consumer_secret|-tcs <secret>: define Twitter consumer secret
+    --twitter_access_token|-tat <token>: define Twitter access token
+    --twitter_access_token_secret|tats <token_secret>: define Twitter access token secret
+    --mastodon_instance|-mi <instance>: define Mastodon instance
+    --mastodon_client_id|-mci <id>: define Mastodon client id
+    --mastodon_client_secret|-mcs <secret>: define Mastodon client secret
+    --mastodon_access_token|-mat <token>: define Mastodon access token
+    --freenode_nickname|-fn <nick>: define Freenode nickname
+    --freenode_password|-fp <passwd>: define Freenode password
+    --freenode_test_channels|-ftc <channel_1>  [--freenode_test_channels|-ftc <channel_2> […--freenode_test_channels|-ftc <channel_n>]]: define Freenode channels
+    --freenode_channels|-fc <channel_1>  [--freenode_channels|-fc <channel_2> […--freenode_channels|-fc <channel_n>]]: define Freenode test channels
+    --revolutionarydate-acab: pretend it is 01:31:20
+    --promptuser-default: define default message when --msgmaker=PromptUser
+USAGE
+ exit 0;
+}
+
+
 no Moose;
 __PACKAGE__->meta->make_immutable(inline_constructor => 0);
 
@@ -274,7 +341,7 @@ App::SpreadRevolutionaryDate::Config - Companion class of L<App::SpreadRevolutio
 
 =head1 VERSION
 
-version 0.09
+version 0.10
 
 =head1 METHODS
 
@@ -302,6 +369,10 @@ Takes one mandatory argument: C<target> as a string in lower case, without any u
 
 Takes one mandatory argument: C<msgmaker> as a string. Returns a hash with configuration options relative to the passed C<msgmaker> argument.
 
+=head2 usage
+
+Prints usage with command line parameters and exits.
+
 =head1 SEE ALSO
 
 =over
